@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2006 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2005-2007 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -18,6 +18,7 @@
 #include "config.h"
 #endif
 
+#include <cstdlib>
 #include "clientbase.h"
 #include "connection.h"
 #include "messagesessionhandler.h"
@@ -51,7 +52,7 @@ namespace gloox
       m_messageSessionHandler( 0 ), m_parser( 0 ),
       m_authError( AuthErrorUndefined ), m_streamError( StreamErrorUndefined ),
       m_streamErrorAppCondition( 0 ), m_idCount( 0 ), m_autoMessageSession( false ),
-      m_fdRequested( false )
+      m_fdRequested( false ), m_disconnectAnnounced( false )
   {
   }
 
@@ -62,7 +63,7 @@ namespace gloox
       m_messageSessionHandler( 0 ), m_parser( 0 ),
       m_authError( AuthErrorUndefined ), m_streamError( StreamErrorUndefined ),
       m_streamErrorAppCondition( 0 ), m_idCount( 0 ), m_autoMessageSession( false ),
-      m_fdRequested( false )
+      m_fdRequested( false ), m_disconnectAnnounced( false )
   {
   }
 
@@ -100,6 +101,7 @@ namespace gloox
       m_connection->setClientCert( m_clientKey, m_clientCerts );
 #endif
 
+    m_disconnectAnnounced = false;
     int ret = m_connection->connect();
     if( ret == StateConnected )
     {
@@ -177,7 +179,7 @@ namespace gloox
     if( m_connection )
     {
       if( reason != ConnStreamError )
-       send ( "</stream:stream>" );
+        send( "</stream:stream>" );
       if( reason == ConnUserDisconnected )
         m_streamError = StreamErrorUndefined;
       m_connection->disconnect( reason );
@@ -575,6 +577,19 @@ namespace gloox
     }
   }
 
+  void ClientBase::removeIDHandler( IqHandler *ih )
+  {
+    IqTrackMap::iterator t;
+    IqTrackMap::iterator it = m_iqIDHandlers.begin();
+    while( it != m_iqIDHandlers.end() )
+    {
+      t = it;
+      ++it;
+      if( ih == (*t).second.ih )
+        m_iqIDHandlers.erase( t );
+    }
+  }
+
   void ClientBase::removeIqHandler( const std::string& xmlns )
   {
     if( !xmlns.empty() )
@@ -671,6 +686,11 @@ namespace gloox
 
   void ClientBase::notifyOnDisconnect( ConnectionError e )
   {
+    if( m_disconnectAnnounced )
+      return;
+
+    m_disconnectAnnounced = true;
+
     ConnectionListenerList::const_iterator it = m_connectionListeners.begin();
     for( ; it != m_connectionListeners.end(); ++it )
     {
@@ -805,9 +825,4 @@ namespace gloox
     }
   }
 
-  void ClientBase::cleanup()
-  {
-
-  }
-
 }
