@@ -1,5 +1,5 @@
 /*
-  Copyright (c) 2005-2006 by Jakob Schroeter <js@camaya.net>
+  Copyright (c) 2005-2007 by Jakob Schroeter <js@camaya.net>
   This file is part of the gloox library. http://camaya.net/gloox
 
   This software is distributed under a license. The full license
@@ -97,7 +97,7 @@ namespace gloox
       /**
        * Use this periodically to receive data from the socket and to feed the parser. You need to use
        * this only if you chose to connect in non-blocking mode.
-       * @param timeout The timeout in seconds to use for select. Default of -1 means blocking
+       * @param timeout The timeout in microseconds to use for select. Default of -1 means blocking
        * until data was available.
        * @return The state of the connection.
        */
@@ -259,6 +259,15 @@ namespace gloox
        */
       void trackID( IqHandler *ih, const std::string& id, int context );
 
+      /**
+       * Removes the given IqHandler from the list of handlers of pending operations, added
+       * using trackID(). Necessary, for example, when closing a GUI elelemnt that has an
+       * operation pending.
+       * @param ih The IqHandler to remove.
+       * @since 0.8.7
+       */
+      void removeIDHandler( IqHandler *ih );
+
       /**
        * Registers @c mh as object that receives Message stanza notifications.
        * @param mh The object to receive Message stanza notifications.
@@ -473,7 +482,7 @@ namespace gloox
       virtual void handleStartNode() = 0;
       virtual bool handleNormalNode( Stanza *stanza ) = 0;
       virtual void rosterFilled() = 0;
-      virtual void cleanup();
+      virtual void cleanup() {};
       void handleStreamError( Stanza *stanza );
 
       void notifyIqHandlers( Stanza *stanza );
@@ -537,7 +546,7 @@ namespace gloox
       int m_idCount;
       bool m_autoMessageSession;
       bool m_fdRequested;
-
+      bool m_disconnectAnnounced;
   };
 
 }
