@@ -40,8 +40,10 @@
 
 #include <time.h>
 
+#include <cstdlib>
 #include <string>
 #include <sstream>
+#include <algorithm>
 
 namespace gloox
 {
@@ -272,14 +274,14 @@ namespace gloox
       error = true;
     else if( t < time( 0 ) )
       m_certInfo.status |= CertExpired;
-    m_certInfo.date_from = t;
+    m_certInfo.date_to = t;
 
     t = (int)gnutls_x509_crt_get_activation_time( cert[0] );
     if( t == -1 )
       error = true;
     else if( t > time( 0 ) )
       m_certInfo.status |= CertNotActive;
-    m_certInfo.date_to = t;
+    m_certInfo.date_from = t;
 
     char name[64];
     size_t nameSize = sizeof( name );
@@ -952,6 +954,9 @@ printf( "maximumMessage: %ld\n", m_streamSizes.cbMaximumMessage );
 
   bool Connection::dataAvailable( int timeout )
   {
+    if( m_socket < 0 )
+      return true;
+
 #ifdef HAVE_TLS
     if( tls_dataAvailable() )
     {
@@ -965,10 +970,10 @@ printf( "maximumMessage: %ld\n", m_streamSizes.cbMaximumMessage );
     FD_ZERO( &fds );
     FD_SET( m_socket, &fds );
 
-    tv.tv_sec = timeout / 1000;
-    tv.tv_usec = timeout % 1000;
+    tv.tv_sec = timeout / 1000000;
+    tv.tv_usec = timeout % 1000000;
 
-    if( select( m_socket + 1, &fds, 0, 0, timeout == -1 ? 0 : &tv ) >= 0 )
+    if( select( m_socket + 1, &fds, 0, 0, timeout == -1 ? 0 : &tv ) > 0 )
     {
       return FD_ISSET( m_socket, &fds ) ? true : false;
     }
@@ -1013,11 +1018,15 @@ printf( "maximumMessage: %ld\n", m_streamSizes.cbMaximumMessage );
     if( size < 0 )
     {
       // error
+      m_secure = false;
+      cleanup();
       return ConnIoError;
     }
     else if( size == 0 )
     {
       // connection closed
+      m_secure = false;
+      cleanup();
       return ConnUserDisconnected;
     }
 
